"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const util = require("util");
const resources_1 = require("../resources");
const constants_1 = require("../constants");
const path = require("path");
const fs = require("fs");
const osHelpers_1 = require("../osHelpers");
const os = require("os");
const constants_2 = require("../constants");
/**
 * The base service class for extension services.
 */
class ExtensionService {
    constructor(context) {
        this.context = context;
    }
    activate() {
    }
    /**
     * Registers a command which can only run if a folder is opened in VSCode.
     */
    registerFolderCommand(commandName, callback) {
        var disposable = vscode.commands.registerCommand(commandName, (...args) => {
            if (!vscode.workspace.rootPath) {
                return this.showNoRootPathWarning();
            }
            return callback(...args);
        });
        this.registerForDisposal(disposable);
    }
    showNoRootPathWarning() {
        return vscode.window.showWarningMessage(util.format(resources_1.default.noRootPathMessage, constants_1.AlExtensionName), resources_1.default.openFolderAction)
            .then(choice => {
            if (choice === resources_1.default.openFolderAction) {
                vscode.commands.executeCommand('vscode.openFolder');
            }
        });
    }
    getServerPath() {
        let alConfig = vscode.workspace.getConfiguration(constants_2.AlLanguageId);
        let serverPath = alConfig.get('editorServicesPath');
        if (!path.isAbsolute(serverPath)) {
            serverPath = this.context.asAbsolutePath(serverPath);
        }
        if (!fs.existsSync(path.join(serverPath, 'Microsoft.Dynamics.Nav.EditorServices.Host.exe'))) {
            serverPath = path.join(serverPath, os.platform());
        }
        serverPath = path.join(serverPath, 'Microsoft.Dynamics.Nav.EditorServices.Host');
        if (osHelpers_1.isWindows()) {
            serverPath += ".exe";
        }
        return serverPath;
    }
    registerForDisposal(disposable) {
        this.context.subscriptions.push(disposable);
    }
}
exports.ExtensionService = ExtensionService;
//# sourceMappingURL=extensionService.js.map