"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const resources_1 = require("../resources");
const extensionService_1 = require("./extensionService");
const status_1 = require("../status");
const constants_1 = require("../constants");
const serverProxy_1 = require("./serverProxy");
class OpenPageDesignerService extends extensionService_1.ExtensionService {
    constructor(context, launchService, languageServerClient, buildService) {
        super(context);
        this.launchService = launchService;
        this.languageServerClient = languageServerClient;
        this.buildService = buildService;
        this.serverProxy = new serverProxy_1.ServerProxy(languageServerClient);
    }
    activate() {
        this.registerFolderCommand('al.openPageDesigner', () => this.openPageDesigner());
    }
    openPageDesigner() {
        return this.launchService.getLaunchConfiguration().then((c) => this.sendOpenPageDesignerRequest(c));
    }
    sendOpenPageDesignerRequest(launchConfig) {
        if (!launchConfig.startupObjectId) {
            status_1.outputChannel.show();
            status_1.outputChannel.appendLine(resources_1.default.startupObjectIdMissing);
            return;
        }
        var alConfig = vscode.workspace.getConfiguration(constants_1.AlLanguageId);
        var params = {
            env: alConfig.get("env")
        };
        this.buildService.packageContainer(vscode.workspace.rootPath)
            .then(() => this.serverProxy.sendRequest(launchConfig, constants_1.AlOpenPageDesignerRequest, params));
    }
}
exports.OpenPageDesignerService = OpenPageDesignerService;
//# sourceMappingURL=openPageDesignerService.js.map